/**
 * 
 */
package net.bigeon.gclc.process;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;

import java.io.IOException;

import org.junit.Test;

import net.bigeon.gclc.exception.CommandRunException;
import net.bigeon.gclc.exception.CommandRunExceptionType;
import net.bigeon.gclc.utils.PipedConsoleInput;
import net.bigeon.gclc.utils.PipedConsoleOutput;

/** @author Emmanuel Bigeon */
public class ForkTaskTest {

    @Test
    public void testGenericForkTask() throws IOException {
        final ForkTask task = new ForkTask(5) {

            @Override
            public String getName() {
                return "name";
            }

            @Override
            protected void doRun() throws CommandRunException {
                String msg;
                try {
                    msg = in.prompt();
                } catch (final IOException e) {
                    throw new CommandRunException(CommandRunExceptionType.INTERACTION,
                            "Unable to prompt user");
                }
                if ("ok".equals(msg)) {
                    out.println("Message");
                } else {
                    out.println("fail");
                }
            }
        };

        final Thread execThread = new Thread(task);
        execThread.start();

        try (PipedConsoleOutput pco = new PipedConsoleOutput();
                PipedConsoleInput pci = new PipedConsoleInput(null)) {
            pci.type("ok");
            while (!pco.available()) {
                task.join(pco, pci, 1000);
            }
            assertEquals("Execution should work", "Message", pco.readNextLine());
        }
        assertFalse("Running state should be updated by task on its completion",
                task.isRunning());
    }

}
